% DistPoint2EpipolarLine  Compute the point-to-epipolar-line distance
%
%   Input:
%   - F(3,3): Fundamental matrix
%   - x1(3,NumPoints): homogeneous coords of the observed points in image 1
%   - x2(3,NumPoints): homogeneous coords of the observed points in image 2
%
%   Output:
%   - cost: sum of squared distance from points to epipolar lines
%           normalized by the number of point coordinates


function cost = DistPoint2EpipolarLine(F,x1,x2)

NumPoints = size(x1,2);
homog_points = [x1, x2];
epi_lines = [F.'*x2, F*x1];

denom = epi_lines(1,:).^2 + epi_lines(2,:).^2;
cost = sqrt( sum( (sum(epi_lines.*homog_points,1).^2)./denom ) / NumPoints );
