% FundamentalEightPoint_Normalized  The normalized DLT 8-point algorithm for F (fundamental matrix) 
%
% The eight-point algorithm for the fundamental matrix with a posteriori
% enforcement of the singularity constraint (det(F)=0).
% It includes image point "normalization"
%
% Reference: "Multiple View Geometry" (Hartley & Zisserman 2000), Sect. 10.1 page 265.
%
% Input: point correspondences
%  - x1(3,N): homogeneous coordinates of 2-D points in image 1
%  - x2(3,N): homogeneous coordinates of 2-D points in image 2
%
% Output:
%  - F(3,3) : fundamental matrix
%  - cost   : fitting cost
%
% See also FundamentalEightPoint


function [F,cost] = FundamentalEightPoint_Normalized(x1,x2)

% 1. Normalization
% Normalise each set of points so that the origin
% is at centroid and mean distance from origin is sqrt(2).
% normalise2dpts also ensures the scale parameter is 1.
[x1_nh,T1] = normalise2dpts(x1);
[,] = ; % Normalize points in image 2
    
% 2. Linear solution
% Use the 8-point algorithm on the normalized data
% Return Fn and fitting cost

% 3. Undo the normalization
F = (T2.') * Fn * T1;
F = F / norm(F);
