% COST_FUNDMAT_SAMPSON  Compute squared distance from each point correspondence to the fundamental matrix
%
% Evaluate the first order approximation of the geometric error
% (Sampson distance) of the fit of a fundamental matrix with respect to a
% set of matched points.
% Reference: "Multiple View Geometry" Hartley & Zisserman 2000. page 271
%
% Input: 
%   - F(3,3): fundamental matrix
%   - x1(3,N): homogeneous coordinates of points in image 1
%   - x2(3,N): homogeneous coordinates of points in image 2
%
% Output: 
%   - SqrdErrVec(N,1): vector of squared Sampson distances. One for each
%   point correspondence.


function dSquared = SqrdErrVec_fundmat_Sampson(F,x1,x2)

a1 = ; % epipolar lines in image 2. One column per line (homog coords)
a2 = ; % epipolar lines in image 1. One column per line (homog coords)

Numer = sum( x2.*a1, 1).^2; 
Denom = a1(1,:).^2 + a1(2,:).^2 + a2(1,:).^2 + a2(2,:).^2;
dSquared = Numer./Denom;
dSquared = dSquared(:); % column vector. Array of squared distance values.
