% Lab session #3  Two-view geometry
% RANSAC for line estimation
%
% Guillermo Gallego

% You can run the entire script by pressing the F5 key
% Or, you can evaluate one or more lines by selecting them and pressing the
% F9 key (without needing to copy them to the prompt in the Command Window)

clear all % clear all variables in the workspace
close all % close all figures
clc       % clear the command window


%% RANSAC for line estimation

% Generate points on a line
L = randn(3,1); % Homogeneous coords of line l=(a,b,c) in a*x + b*y + c = 0
NumPoints = 100;
x = 10*randn(NumPoints,1);
y = -(L(1)*x+L(3)) / L(2);
points = [x,y].';

% Add noise
sigma = 0.3;
noise = sigma*randn(2,NumPoints);
noisy_points = points + noise;

% Generate outliers
obs_points = [noisy_points, 10*randn(2,round(NumPoints*0.2))];

% Display points
figure, plot(obs_points(1,:), obs_points(2,:),'o');

% Estimate 2-D line using RANSAC
fittingfn = @line_fitting;
distfn    = @line_dist;
degenfn   = @line_isdegenerate;
s = 2; % two points define a line
t = 0.5; % pixel unit. Inlier classification threshold
feedback_print = 1;
[L_est, inliers] = ransac(obs_points, fittingfn, distfn, degenfn, s, t, feedback_print); % more args: maxDataTrials, MaxTrials);

% Print number of points and number of inliers
disp(['Number of points = ' num2str(size(obs_points,2))]);
disp(['Number of inliers = ' num2str(numel(inliers))]);

% Display fitted line or corrected points
hold on 
y = -(L_est(1)*obs_points(1,:)+L_est(3)) / L_est(2);
vert_corr_points = [obs_points(1,:); y];
plot(vert_corr_points(1,:),vert_corr_points(2,:),'m');
hold off

% Some error measure between true and estimated line
SphDistance(L,L_est)
