% E2TXR  Decompose essential matrix into motion parameters: translation and rotation
%
% Given an essential matrix, compute the camera motion, i.e.,  R and T such
% that E ~ T_x R
% 
% Input:
%   - E(3,3) : Essential matrix
%
% Output:
%   - R(3,3,2) : all possible rotations
%   - T(3,1)   : unit vector with the translation information


function [R,T] = DecomposeEssentialMatrix(E)

[U,D,V] = svd(E);

% This function assumes that the two non-zero singular values in D are
% equal. Otherwise it is enforced.

% Translation
T = U(:,3);

% Rotations
W = [0 -1 0; 1 0 0; 0 0 1];
R(:,:,1) = U*W*V.';
R(:,:,2) = U*W.'*V.';
if det(R(:,:,1))<0
    R=-R;
end

return


%% Check
SphDistance(Cross2Matrix(T)*(R(:,:,1)),E)
SphDistance(Cross2Matrix(T)*(R(:,:,2)),E)
