% DEPTHOFPOINTS  Depth of given 3-D points with respect to each camera
%
% Computes the depth of a set of 3-D points with respect to the cameras
% given by their projection matrices in a projective reconstruction.  
% Equation in $5.2.3, book Multiple View Geometry 2000 by Hartley-Zisserman
%
%               sign(det(M))*w  
% depth(X;P) = ----------------
%                T*||m^{3}||
%
% Input:
%  - P(3,4,ncam) : projection matrices
%  - Q(4,npoints): homogeneous coordinates of space points
%
% Output:
%  - d(ncam,npoints): depth of the space points with respect to the cameras


function d = DepthOfPoints(P,Q)

ncam = size(P,3);      % Number of cameras
npoints = size(Q,2);   % Number of 3D points

d = zeros(ncam,npoints);

for k=1:ncam
    q = P(:,:,k)*Q;
    cte = sign(det(P(1:3,1:3,k)))/norm(P(3,1:3,k));
    d(k,:) = cte*q(3,:)./Q(4,:);
end