% Lab session #3  Two-view geometry
% Structure from motion: from images to 3-D scene
%
% Guillermo Gallego

% You can run the entire script by pressing the F5 key
% Or, you can evaluate one or more lines by selecting them and pressing the
% F9 key (without needing to copy them to the prompt in the Command Window)

clear all % clear all variables in the workspace
close all % close all figures
clc       % clear the command window


%% Dataset. Load a pair of images.
% image numbers to construct the image-pair to establish correspondences in
imgNo1 = 4;
imgNo2 = 5;

% Intrinsic parameter matrix - calibrated, do not change
K = [1379.74 0 760.35
    0 1382.08 503.41
    0 0 1 ];

% Read images
im1 = imread(['fountain/00' num2str(imgNo1,'%02d') '.jpg']);
im2 = imread(['fountain/00' num2str(imgNo2,'%02d') '.jpg']);


%% Detect and extract points
display = 1;        % set to 1 if you want to display the matches

% BRISK features.
minContrast = 0.02; % BRISK parameter

% Detect and describe features in image 1
im1_g = rgb2gray(im1); % Convert color image to grayscale
% Detect features
points = detectBRISKFeatures(im1_g, 'MinContrast', minContrast);
% Describe neighborhood of features
[featDescriptors1,validPoints1] = extractFeatures(im1_g,points,'Method','BRISK');
% (validPoints are those whose neighborhood is fully contained in the image)
disp(['Detected ' num2str(validPoints1.Count) ' BRISK keypoints in first image.']);

figure;
subplot(1,2,1)
imshow(im1,[]);
hold on
plot(validPoints1.selectStrongest(100),'showOrientation',true);
title('Image 1')

% Detect and describe features for second image.
im2_g = rgb2gray(im2);
points = detectBRISKFeatures(im2_g, 'MinContrast', minContrast);
[featDescriptors2,validPoints2] = extractFeatures(im2_g,points,'Method','BRISK');
disp(['Detected ' num2str(validPoints2.Count) ' BRISK keypoints in second image.']);

subplot(1,2,2)
imshow(im2,[]);
hold on
plot(validPoints2.selectStrongest(100),'showOrientation',true);
title('Image 2')


%% Match the features
% If given binary descriptors, Hamming distance is used to compute similarity.
[indexPairs, matchmetric] = matchFeatures(featDescriptors1,featDescriptors2);
NumMatches = size(indexPairs,1);
disp(['Found ' num2str(NumMatches) ' BRISK matches']);

% Retrieve the locations of corresponding points for each image.
matchedPoints1 = validPoints1(indexPairs(:, 1), :);
matchedPoints2 = validPoints2(indexPairs(:, 2), :);

% Homogeneous coordinates of matched image points
% (in preparation for the next processing step)
x1_pix = [matchedPoints1.Location, ones(NumMatches,1)].';
x2_pix = [matchedPoints2.Location, ones(NumMatches,1)].';

% Display raw matches (unfiltered by RANSAC) between both images
if (display)
    figure, imshow([im1 im2],[]); % Display images side by side
    
    % Using a 'for' loop
    hold on
    for i = 1:NumMatches
        color = rand(1,3);
        % Draw a line between corresponding points
        plot([x1_pix(1,i), x2_pix(1,i) + size(im1,2)], ...
            [x1_pix(2,i), x2_pix(2,i)], '-', 'Color', color);
    end
    hold off
    title('Raw correspondences. Not filtered by RANSAC')
    
    %     % Alternative code:
    %     % Draw lines between corresponding points
    %     figure, imshow([im1 im2],[]); % Display images side by side
    %     % 2 x N.  X and Y coordinates of the end points of each line.
    %     X = [x1_pix(1,:); x2_pix(1,:) + size(im1,2)];
    %     Y = [x1_pix(2,:); x2_pix(2,:)];
    %     line(X,Y) % one line per column of X and Y
    %     title('Raw correspondences. Not filtered by RANSAC')
end

% Variables that are no longer in use (could be cleared):
% indexPairs, matchmetric, validPoints1, validPoints2, featDescriptors1,
% featDescriptors2, points, im1_g, im2_g.


%% Estimate fundamental matrix F using RANSAC on the matched points

% Similarity transformation to improve numerical conditioning
% sn = 1; % No scaling
sn = ( size(im1,1) + size(im1,2) ) / 2.;
Tn = diag([1/sn,1/sn,1]);
x1n = Tn * x1_pix;
x2n = Tn * x2_pix;

% Call RANSAC function
obs_points = [x1n; x2n]; % Adapt data: combine points in both images into a single variable
fittingfn = @;
distfn    = @;
degenfn   = @;
s = ; % number of points that are used to define a fundamental matrix
t = 2 / sn; % [pixel] unit. Inlier classification threshold
feedback = 1;

% Call ransac function. Use as an example the code provided for lie fitting
% Use obs_points as an input argument to ransac()


disp(['Number of points = ' num2str(size(obs_points,2))]);
NumInliers = numel(inliers);
disp(['Number of inliers = ' num2str(NumInliers)]);

% Coordinates of inlier points
x1n = x1n(:,inliers);
x2n = x2n(:,inliers);

dSquared = SqrdErrVec_fundmat_Sampson(Fn, x1n, x2n);
cost_Sampson = sn * sqrt(sum( dSquared )) / sqrt(NumInliers); % pixels per coordinate
disp(['cost_Sampson [pixels] = ', num2str(cost_Sampson)]);


%% Re-estimate F using the inliers
disp(' ');
disp('Re-estimate F using the inliers:');
Fn = FundamentalEightPoint(x1n,x2n);
dSquared = SqrdErrVec_fundmat_Sampson(Fn, x1n, x2n);
cost_Sampson = sn * sqrt(sum( dSquared )) / sqrt(NumInliers); % pixels per coordinate
disp(['cost_Sampson [pixels] = ', num2str(cost_Sampson)]);


%% Display filtered correspondences, i.e., the inliers
x1_pix_in = x1_pix(:,inliers);
x2_pix_in = x2_pix(:,inliers);

% Draw matches filtered by the epipolar constraint, i.e., draw inliers
figure, imshow([im1 im2],[]); % Display images side by side
hold on
for i = 1:NumInliers
    
    
end
hold off


%% Obtain essential matrix E from F and intrinsics (K1,K2)
K1 = K; K2 = K; % Given with the dataset
F = Fn;         % Estimated fundamental matrix

% Transform the intrinsic parameters
K1 = Tn * K1;
K2 = Tn * K2;

E = ;
disp('Singular values of the Essential Matrix:')
svd(E) % The two non-zero singular values should be equal (up to noise)

% Obtain extrinsic parameters (R,t) from E
[Rots,T] = DecomposeEssentialMatrix(E);
% This returns two possible rotations. NEED TO CHOOSE THE CORRECT ONE (iCam)

% Build finite camera projection matrices and triangulate inlier points
P1 = K1 * eye(3,4); % Projection matrix of camera 1

iCamFront = 1;
NumPositiveDepths_best = 0;
for iRot = 1:2
    Rot_test = Rots(:,:,iRot);
    
    for iSignT = 1:2
        T_test = T * (-1)^iSignT;
        
        P2 = K2 * [Rot_test, T_test];
        
        % Triangulate points x1n,x2n using P1 and P2
        
        
        X_euc_test = X(1:3,:)./repmat(X(4,:),3,1); % (Euclidize) Multiple points
        
        depths = DepthOfPoints(cat(3,P1,P2),X);
        NumPositiveDepths = ; % Compute the number of positive depths
        
        %PointsInFrontOfBothCameras = all( d(:)>0 );
        if (NumPositiveDepths > NumPositiveDepths_best)
            % Keep the rotation that gives points in front of both cameras
            Rot_best = Rot_test;
            T_best = T_test;
            X_euc = X_euc_test;
            NumPositiveDepths_best = NumPositiveDepths;
        end
    end
end

% Build projection matrix (of camera 2) with correct camera motion
P2 = ;


%% Visualize the 3-D scene
figure,
subplot(1,3,1)

% plot 3-D pooints X_euc
plot3( , 'o');

% Display camera poses
% Camera 2
Center2 = -Rot_best.'*T_best;
plotCoordinateFrame(Rot_best.',Center2, 0.8);
text(Center2(1)-0.1, Center2(2)-0.1, Center2(3)-0.1,'Cam 2','fontsize',10,'color','k','FontWeight','bold');

% Camera 1


text(,'Cam 1','fontsize',10,'color','k','FontWeight','bold');

axis equal
rotate3d on;
grid


%% Display matched points filtered by the epipolar constraint, i.e., inliers
% figure;
subplot(1,3,2)
imshow(im1,[]);
hold on
plot(x1_pix_in(1,:), x1_pix_in(2,:), 'ys');
title('Image 1')

% Same thing, but with the second image
;

